@@ -2,21 +2,22 @@ package machine
 
 import (
 	"context"
+	"sigs.k8s.io/yaml"
 	"testing"
 	"time"
 
 	bmoapis "github.com/metal3-io/baremetal-operator/pkg/apis"
 	bmh "github.com/metal3-io/baremetal-operator/pkg/apis/metal3/v1alpha1"
-	bmv1alpha1 "github.com/metal3-io/cluster-api-provider-baremetal/pkg/apis/baremetal/v1alpha1"
-	clusterapis "github.com/openshift/cluster-api/pkg/apis"
-	machinev1 "github.com/openshift/cluster-api/pkg/apis/machine/v1beta1"
-	clustererror "github.com/openshift/cluster-api/pkg/controller/error"
+	bmv1alpha1 "github.com/openshift/cluster-api-provider-baremetal/pkg/apis/baremetal/v1alpha1"
+	machinev1beta1 "github.com/openshift/machine-api-operator/pkg/apis/machine/v1beta1"
+	machineapierrors "github.com/openshift/machine-api-operator/pkg/controller/machine"
 	corev1 "k8s.io/api/core/v1"
+	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
+	"k8s.io/apimachinery/pkg/types"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	fakeclient "sigs.k8s.io/controller-runtime/pkg/client/fake"
-	"sigs.k8s.io/yaml"
 )
 
 const (
@@ -24,6 +25,7 @@ const (
 	testImageChecksumURL        = "http://172.22.0.1/images/rhcos-ootpa-latest.qcow2.md5sum"
 	testUserDataSecretName      = "worker-user-data"
 	testUserDataSecretNamespace = "myns"
+	testRemediationNamespace    = "remediationNs"
 )
 
 func TestChooseHost(t *testing.T) {
@@ -40,7 +42,7 @@ func TestChooseHost(t *testing.T) {
 				Name:       "someothermachine",
 				Namespace:  "myns",
 				Kind:       "Machine",
-				APIVersion: machinev1.SchemeGroupVersion.String(),
+				APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 			},
 		},
 	}
@@ -60,7 +62,7 @@ func TestChooseHost(t *testing.T) {
 				Name:       "machine1",
 				Namespace:  "myns",
 				Kind:       "Machine",
-				APIVersion: machinev1.SchemeGroupVersion.String(),
+				APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 			},
 		},
 	}
@@ -108,111 +110,116 @@ func TestChooseHost(t *testing.T) {
 		})
 
 	testCases := []struct {
-		Machine          machinev1.Machine
+		Machine          machinev1beta1.Machine
 		Hosts            []runtime.Object
 		ExpectedHostName string
 		Config           *bmv1alpha1.BareMetalMachineProviderSpec
 	}{
 		{
 			// should pick host2, which lacks a ConsumerRef
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "machine1",
 					Namespace: "myns",
 				},
 				TypeMeta: metav1.TypeMeta{
 					Kind:       "Machine",
-					APIVersion: machinev1.SchemeGroupVersion.String(),
+					APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 				},
-				Spec: machinev1.MachineSpec{
+				Spec: machinev1beta1.MachineSpec{
 					ProviderSpec: providerSpec,
 				},
 			},
 			Hosts:            []runtime.Object{&host2, &host1},
 			ExpectedHostName: host2.Name,
+			Config:           config,
 		},
 		{
 			// should ignore discoveredHost and pick host2, which lacks a ConsumerRef
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "machine1",
 					Namespace: "myns",
 				},
 				TypeMeta: metav1.TypeMeta{
 					Kind:       "Machine",
-					APIVersion: machinev1.SchemeGroupVersion.String(),
+					APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 				},
 			},
 			Hosts:            []runtime.Object{&discoveredHost, &host2, &host1},
 			ExpectedHostName: host2.Name,
+			Config:           config,
 		},
 		{
 			// should pick host3, which already has a matching ConsumerRef
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "machine1",
 					Namespace: "myns",
 				},
 				TypeMeta: metav1.TypeMeta{
 					Kind:       "Machine",
-					APIVersion: machinev1.SchemeGroupVersion.String(),
+					APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 				},
-				Spec: machinev1.MachineSpec{
+				Spec: machinev1beta1.MachineSpec{
 					ProviderSpec: providerSpec,
 				},
 			},
 			Hosts:            []runtime.Object{&host1, &host3, &host2},
 			ExpectedHostName: host3.Name,
+			Config:           config,
 		},
 		{
 			// should not pick a host, because two are already taken, and the third is in
 			// a different namespace
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "machine2",
 					Namespace: "myns",
 				},
 				TypeMeta: metav1.TypeMeta{
 					Kind:       "Machine",
-					APIVersion: machinev1.SchemeGroupVersion.String(),
+					APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 				},
-				Spec: machinev1.MachineSpec{
+				Spec: machinev1beta1.MachineSpec{
 					ProviderSpec: providerSpec,
 				},
 			},
 			Hosts:            []runtime.Object{&host1, &host3, &host4},
 			ExpectedHostName: "",
+			Config:           config,
 		},
 		{
 			// Can choose hosts with a label, even without a label selector
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "machine1",
 					Namespace: "myns",
 				},
 				TypeMeta: metav1.TypeMeta{
 					Kind:       "Machine",
-					APIVersion: machinev1.SchemeGroupVersion.String(),
+					APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 				},
-				Spec: machinev1.MachineSpec{
+				Spec: machinev1beta1.MachineSpec{
 					ProviderSpec: providerSpec,
 				},
 			},
 			Hosts:            []runtime.Object{&host_with_label},
 			ExpectedHostName: host_with_label.Name,
+			Config:           config,
 		},
 		{
 			// Choose the host with the right label
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "machine1",
 					Namespace: "myns",
 				},
 				TypeMeta: metav1.TypeMeta{
 					Kind:       "Machine",
-					APIVersion: machinev1.SchemeGroupVersion.String(),
+					APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 				},
-				Spec: machinev1.MachineSpec{
+				Spec: machinev1beta1.MachineSpec{
 					ProviderSpec: providerSpec2,
 				},
 			},
@@ -222,16 +229,16 @@ func TestChooseHost(t *testing.T) {
 		},
 		{
 			// No host that matches required label
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "machine1",
 					Namespace: "myns",
 				},
 				TypeMeta: metav1.TypeMeta{
 					Kind:       "Machine",
-					APIVersion: machinev1.SchemeGroupVersion.String(),
+					APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 				},
-				Spec: machinev1.MachineSpec{
+				Spec: machinev1beta1.MachineSpec{
 					ProviderSpec: providerSpec3,
 				},
 			},
@@ -241,16 +248,16 @@ func TestChooseHost(t *testing.T) {
 		},
 		{
 			// Host that matches a matchExpression
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "machine1",
 					Namespace: "myns",
 				},
 				TypeMeta: metav1.TypeMeta{
 					Kind:       "Machine",
-					APIVersion: machinev1.SchemeGroupVersion.String(),
+					APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 				},
-				Spec: machinev1.MachineSpec{
+				Spec: machinev1beta1.MachineSpec{
 					ProviderSpec: providerSpec4,
 				},
 			},
@@ -260,16 +267,16 @@ func TestChooseHost(t *testing.T) {
 		},
 		{
 			// No Host available that matches a matchExpression
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "machine1",
 					Namespace: "myns",
 				},
 				TypeMeta: metav1.TypeMeta{
 					Kind:       "Machine",
-					APIVersion: machinev1.SchemeGroupVersion.String(),
+					APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 				},
-				Spec: machinev1.MachineSpec{
+				Spec: machinev1beta1.MachineSpec{
 					ProviderSpec: providerSpec4,
 				},
 			},
@@ -279,16 +286,16 @@ func TestChooseHost(t *testing.T) {
 		},
 		{
 			// No host chosen given an Invalid match expression
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "machine1",
 					Namespace: "myns",
 				},
 				TypeMeta: metav1.TypeMeta{
 					Kind:       "Machine",
-					APIVersion: machinev1.SchemeGroupVersion.String(),
+					APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 				},
-				Spec: machinev1.MachineSpec{
+				Spec: machinev1beta1.MachineSpec{
 					ProviderSpec: providerSpec5,
 				},
 			},
@@ -307,11 +314,13 @@ func TestChooseHost(t *testing.T) {
 		if err != nil {
 			t.Errorf("%v", err)
 		}
-		cfg := tc.Config
-		if cfg == nil {
-			cfg = config
+		pspec, err := yaml.Marshal(&tc.Config)
+		if err != nil {
+			t.Logf("could not marshal BareMetalMachineProviderSpec: %v", err)
+			t.FailNow()
 		}
-		result, err := actuator.chooseHost(context.TODO(), &tc.Machine, cfg)
+		tc.Machine.Spec.ProviderSpec = machinev1beta1.ProviderSpec{Value: &runtime.RawExtension{Raw: pspec}}
+		result, err := actuator.chooseHost(context.TODO(), &tc.Machine)
 		if tc.ExpectedHostName == "" {
 			if result != nil {
 				t.Error("found host when none should have been available")
@@ -385,7 +394,7 @@ func TestSetHostSpec(t *testing.T) {
 						Name:       "machine1",
 						Namespace:  "myns",
 						Kind:       "Machine",
-						APIVersion: machinev1.SchemeGroupVersion.String(),
+						APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 					},
 				},
 			},
@@ -410,7 +419,7 @@ func TestSetHostSpec(t *testing.T) {
 						Name:       "machine1",
 						Namespace:  "myns",
 						Kind:       "Machine",
-						APIVersion: machinev1.SchemeGroupVersion.String(),
+						APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 					},
 					Image: &bmh.Image{
 						URL:      testImageURL + "test",
@@ -429,12 +438,12 @@ func TestSetHostSpec(t *testing.T) {
 		t.Run(tc.Scenario, func(t *testing.T) {
 			// test data
 			config, providerSpec := newConfig(t, tc.UserDataNamespace, map[string]string{}, []bmv1alpha1.HostSelectorRequirement{})
-			machine := machinev1.Machine{
+			machine := machinev1beta1.Machine{
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "machine1",
 					Namespace: "myns",
 				},
-				Spec: machinev1.MachineSpec{
+				Spec: machinev1beta1.MachineSpec{
 					ProviderSpec: providerSpec,
 				},
 			}
@@ -529,13 +538,13 @@ func TestExists(t *testing.T) {
 
 	testCases := []struct {
 		Client      client.Client
-		Machine     machinev1.Machine
+		Machine     machinev1beta1.Machine
 		Expected    bool
 		FailMessage string
 	}{
 		{
 			Client: c,
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				ObjectMeta: metav1.ObjectMeta{
 					Annotations: map[string]string{
 						HostAnnotation: "myns/somehost",
@@ -547,7 +556,7 @@ func TestExists(t *testing.T) {
 		},
 		{
 			Client: c,
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				ObjectMeta: metav1.ObjectMeta{
 					Annotations: map[string]string{
 						HostAnnotation: "myns/wrong",
@@ -559,7 +568,7 @@ func TestExists(t *testing.T) {
 		},
 		{
 			Client:      c,
-			Machine:     machinev1.Machine{},
+			Machine:     machinev1beta1.Machine{},
 			Expected:    false,
 			FailMessage: "found host even though annotation not present",
 		},
@@ -573,7 +582,7 @@ func TestExists(t *testing.T) {
 			t.Error(err)
 		}
 
-		result, err := actuator.Exists(context.TODO(), nil, &tc.Machine)
+		result, err := actuator.Exists(context.TODO(), &tc.Machine)
 		if err != nil {
 			t.Error(err)
 		}
@@ -596,13 +605,13 @@ func TestGetHost(t *testing.T) {
 
 	testCases := []struct {
 		Client        client.Client
-		Machine       machinev1.Machine
+		Machine       machinev1beta1.Machine
 		ExpectPresent bool
 		FailMessage   string
 	}{
 		{
 			Client: c,
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				ObjectMeta: metav1.ObjectMeta{
 					Annotations: map[string]string{
 						HostAnnotation: "myns/myhost",
@@ -614,7 +623,7 @@ func TestGetHost(t *testing.T) {
 		},
 		{
 			Client: c,
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				ObjectMeta: metav1.ObjectMeta{
 					Annotations: map[string]string{
 						HostAnnotation: "myns/wrong",
@@ -626,7 +635,7 @@ func TestGetHost(t *testing.T) {
 		},
 		{
 			Client:        c,
-			Machine:       machinev1.Machine{},
+			Machine:       machinev1beta1.Machine{},
 			ExpectPresent: false,
 			FailMessage:   "found host even though annotation not present",
 		},
@@ -652,15 +661,15 @@ func TestGetHost(t *testing.T) {
 
 func TestEnsureAnnotation(t *testing.T) {
 	scheme := runtime.NewScheme()
-	clusterapis.AddToScheme(scheme)
+	machinev1beta1.AddToScheme(scheme)
 
 	testCases := []struct {
-		Machine machinev1.Machine
+		Machine machinev1beta1.Machine
 		Host    bmh.BareMetalHost
 	}{
 		{
 			// annotation exists and is correct
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				ObjectMeta: metav1.ObjectMeta{
 					Annotations: map[string]string{
 						HostAnnotation: "myns/myhost",
@@ -676,7 +685,7 @@ func TestEnsureAnnotation(t *testing.T) {
 		},
 		{
 			// annotation exists but is wrong
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				ObjectMeta: metav1.ObjectMeta{
 					Annotations: map[string]string{
 						HostAnnotation: "myns/wrongvalue",
@@ -692,7 +701,7 @@ func TestEnsureAnnotation(t *testing.T) {
 		},
 		{
 			// annotations are empty
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				ObjectMeta: metav1.ObjectMeta{
 					Annotations: map[string]string{},
 				},
@@ -706,7 +715,7 @@ func TestEnsureAnnotation(t *testing.T) {
 		},
 		{
 			// annotations are nil
-			Machine: machinev1.Machine{},
+			Machine: machinev1beta1.Machine{},
 			Host: bmh.BareMetalHost{
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "myhost",
@@ -725,13 +734,13 @@ func TestEnsureAnnotation(t *testing.T) {
 			t.Error(err)
 		}
 
-		err = actuator.ensureAnnotation(context.TODO(), &tc.Machine, &tc.Host)
+		_, err = actuator.ensureAnnotation(context.TODO(), &tc.Machine, &tc.Host)
 		if err != nil {
 			t.Errorf("unexpected error %v", err)
 		}
 
 		// get the machine and make sure it has the correct annotation
-		machine := machinev1.Machine{}
+		machine := machinev1beta1.Machine{}
 		key := client.ObjectKey{
 			Name:      tc.Machine.Name,
 			Namespace: tc.Machine.Namespace,
@@ -758,7 +767,7 @@ func TestDelete(t *testing.T) {
 	testCases := []struct {
 		CaseName            string
 		Host                *bmh.BareMetalHost
-		Machine             machinev1.Machine
+		Machine             machinev1beta1.Machine
 		ExpectedConsumerRef *corev1.ObjectReference
 		ExpectedResult      error
 	}{
@@ -774,7 +783,7 @@ func TestDelete(t *testing.T) {
 						Name:       "mymachine",
 						Namespace:  "myns",
 						Kind:       "Machine",
-						APIVersion: machinev1.SchemeGroupVersion.String(),
+						APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 					},
 					Image: &bmh.Image{
 						URL: "myimage",
@@ -786,10 +795,10 @@ func TestDelete(t *testing.T) {
 					},
 				},
 			},
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				TypeMeta: metav1.TypeMeta{
 					Kind:       "Machine",
-					APIVersion: machinev1.SchemeGroupVersion.String(),
+					APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 				},
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "mymachine",
@@ -803,9 +812,9 @@ func TestDelete(t *testing.T) {
 				Name:       "mymachine",
 				Namespace:  "myns",
 				Kind:       "Machine",
-				APIVersion: machinev1.SchemeGroupVersion.String(),
+				APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 			},
-			ExpectedResult: &clustererror.RequeueAfterError{},
+			ExpectedResult: &machineapierrors.RequeueAfterError{},
 		},
 		{
 			CaseName: "deprovisioning in progress",
@@ -819,7 +828,7 @@ func TestDelete(t *testing.T) {
 						Name:       "mymachine",
 						Namespace:  "myns",
 						Kind:       "Machine",
-						APIVersion: machinev1.SchemeGroupVersion.String(),
+						APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 					},
 				},
 				Status: bmh.BareMetalHostStatus{
@@ -828,10 +837,10 @@ func TestDelete(t *testing.T) {
 					},
 				},
 			},
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				TypeMeta: metav1.TypeMeta{
 					Kind:       "Machine",
-					APIVersion: machinev1.SchemeGroupVersion.String(),
+					APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 				},
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "mymachine",
@@ -845,9 +854,9 @@ func TestDelete(t *testing.T) {
 				Name:       "mymachine",
 				Namespace:  "myns",
 				Kind:       "Machine",
-				APIVersion: machinev1.SchemeGroupVersion.String(),
+				APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 			},
-			ExpectedResult: &clustererror.RequeueAfterError{RequeueAfter: time.Second * 30},
+			ExpectedResult: &machineapierrors.RequeueAfterError{RequeueAfter: time.Second * 30},
 		},
 		{
 			CaseName: "externally provisioned host should be powered down",
@@ -861,7 +870,7 @@ func TestDelete(t *testing.T) {
 						Name:       "mymachine",
 						Namespace:  "myns",
 						Kind:       "Machine",
-						APIVersion: machinev1.SchemeGroupVersion.String(),
+						APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 					},
 				},
 				Status: bmh.BareMetalHostStatus{
@@ -871,10 +880,10 @@ func TestDelete(t *testing.T) {
 					PoweredOn: true,
 				},
 			},
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				TypeMeta: metav1.TypeMeta{
 					Kind:       "Machine",
-					APIVersion: machinev1.SchemeGroupVersion.String(),
+					APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 				},
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "mymachine",
@@ -888,9 +897,9 @@ func TestDelete(t *testing.T) {
 				Name:       "mymachine",
 				Namespace:  "myns",
 				Kind:       "Machine",
-				APIVersion: machinev1.SchemeGroupVersion.String(),
+				APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 			},
-			ExpectedResult: &clustererror.RequeueAfterError{RequeueAfter: time.Second * 30},
+			ExpectedResult: &machineapierrors.RequeueAfterError{RequeueAfter: time.Second * 30},
 		},
 		{
 			CaseName: "consumer ref should be removed from externally provisioned host",
@@ -904,7 +913,7 @@ func TestDelete(t *testing.T) {
 						Name:       "mymachine",
 						Namespace:  "myns",
 						Kind:       "Machine",
-						APIVersion: machinev1.SchemeGroupVersion.String(),
+						APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 					},
 				},
 				Status: bmh.BareMetalHostStatus{
@@ -914,10 +923,10 @@ func TestDelete(t *testing.T) {
 					PoweredOn: false,
 				},
 			},
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				TypeMeta: metav1.TypeMeta{
 					Kind:       "Machine",
-					APIVersion: machinev1.SchemeGroupVersion.String(),
+					APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 				},
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "mymachine",
@@ -940,7 +949,7 @@ func TestDelete(t *testing.T) {
 						Name:       "mymachine",
 						Namespace:  "myns",
 						Kind:       "Machine",
-						APIVersion: machinev1.SchemeGroupVersion.String(),
+						APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 					},
 				},
 				Status: bmh.BareMetalHostStatus{
@@ -949,10 +958,10 @@ func TestDelete(t *testing.T) {
 					},
 				},
 			},
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				TypeMeta: metav1.TypeMeta{
 					Kind:       "Machine",
-					APIVersion: machinev1.SchemeGroupVersion.String(),
+					APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 				},
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "mymachine",
@@ -975,7 +984,7 @@ func TestDelete(t *testing.T) {
 						Name:       "someoneelsesmachine",
 						Namespace:  "myns",
 						Kind:       "Machine",
-						APIVersion: machinev1.SchemeGroupVersion.String(),
+						APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 					},
 					Image: &bmh.Image{
 						URL: "someoneelsesimage",
@@ -987,10 +996,10 @@ func TestDelete(t *testing.T) {
 					},
 				},
 			},
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				TypeMeta: metav1.TypeMeta{
 					Kind:       "Machine",
-					APIVersion: machinev1.SchemeGroupVersion.String(),
+					APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 				},
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "mymachine",
@@ -1004,7 +1013,7 @@ func TestDelete(t *testing.T) {
 				Name:       "someoneelsesmachine",
 				Namespace:  "myns",
 				Kind:       "Machine",
-				APIVersion: machinev1.SchemeGroupVersion.String(),
+				APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 			},
 		},
 		{
@@ -1015,10 +1024,10 @@ func TestDelete(t *testing.T) {
 					Namespace: "myns",
 				},
 			},
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				TypeMeta: metav1.TypeMeta{
 					Kind:       "Machine",
-					APIVersion: machinev1.SchemeGroupVersion.String(),
+					APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 				},
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "mymachine",
@@ -1032,10 +1041,10 @@ func TestDelete(t *testing.T) {
 		{
 			CaseName: "no host at all, so this is a no-op",
 			Host:     nil,
-			Machine: machinev1.Machine{
+			Machine: machinev1beta1.Machine{
 				TypeMeta: metav1.TypeMeta{
 					Kind:       "Machine",
-					APIVersion: machinev1.SchemeGroupVersion.String(),
+					APIVersion: machinev1beta1.SchemeGroupVersion.String(),
 				},
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "mymachine",
@@ -1060,15 +1069,15 @@ func TestDelete(t *testing.T) {
 			t.Error(err)
 		}
 
-		err = actuator.Delete(context.TODO(), nil, &tc.Machine)
+		err = actuator.Delete(context.TODO(), &tc.Machine)
 
 		var expectedResult bool
 		switch e := tc.ExpectedResult.(type) {
 		case nil:
 			expectedResult = (err == nil)
-		case *clustererror.RequeueAfterError:
-			var perr *clustererror.RequeueAfterError
-			if perr, expectedResult = err.(*clustererror.RequeueAfterError); expectedResult {
+		case *machineapierrors.RequeueAfterError:
+			var perr *machineapierrors.RequeueAfterError
+			if perr, expectedResult = err.(*machineapierrors.RequeueAfterError); expectedResult {
 				expectedResult = (*e == *perr)
 			}
 		}
@@ -1100,7 +1109,7 @@ func TestDelete(t *testing.T) {
 }
 
 func TestConfigFromProviderSpec(t *testing.T) {
-	ps := machinev1.ProviderSpec{
+	ps := machinev1beta1.ProviderSpec{
 		Value: &runtime.RawExtension{
 			Raw: []byte(`{"apiVersion":"baremetal.cluster.k8s.io/v1alpha1","userData":{"Name":"worker-user-data","Namespace":"myns"},"image":{"Checksum":"http://172.22.0.1/images/rhcos-ootpa-latest.qcow2.md5sum","URL":"http://172.22.0.1/images/rhcos-ootpa-latest.qcow2"},"kind":"BareMetalMachineProviderSpec"}`),
 		},
@@ -1133,7 +1142,7 @@ func TestConfigFromProviderSpec(t *testing.T) {
 	}
 }
 
-func newConfig(t *testing.T, UserDataNamespace string, labels map[string]string, reqs []bmv1alpha1.HostSelectorRequirement) (*bmv1alpha1.BareMetalMachineProviderSpec, machinev1.ProviderSpec) {
+func newConfig(t *testing.T, UserDataNamespace string, labels map[string]string, reqs []bmv1alpha1.HostSelectorRequirement) (*bmv1alpha1.BareMetalMachineProviderSpec, machinev1beta1.ProviderSpec) {
 	config := bmv1alpha1.BareMetalMachineProviderSpec{
 		Image: bmv1alpha1.Image{
 			URL:      testImageURL,
@@ -1153,14 +1162,14 @@ func newConfig(t *testing.T, UserDataNamespace string, labels map[string]string,
 		t.Logf("could not marshal BareMetalMachineProviderSpec: %v", err)
 		t.FailNow()
 	}
-	return &config, machinev1.ProviderSpec{
+	return &config, machinev1beta1.ProviderSpec{
 		Value: &runtime.RawExtension{Raw: out},
 	}
 }
 
 func TestUpdateMachineStatus(t *testing.T) {
 	scheme := runtime.NewScheme()
-	clusterapis.AddToScheme(scheme)
+	machinev1beta1.AddToScheme(scheme)
 
 	nic1 := bmh.NIC{
 		IP: "192.168.1.1",
@@ -1172,8 +1181,8 @@ func TestUpdateMachineStatus(t *testing.T) {
 
 	testCases := []struct {
 		Host            *bmh.BareMetalHost
-		Machine         *machinev1.Machine
-		ExpectedMachine machinev1.Machine
+		Machine         *machinev1beta1.Machine
+		ExpectedMachine machinev1beta1.Machine
 	}{
 		{
 			// machine status updated
@@ -1184,15 +1193,15 @@ func TestUpdateMachineStatus(t *testing.T) {
 					},
 				},
 			},
-			Machine: &machinev1.Machine{
+			Machine: &machinev1beta1.Machine{
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "mymachine",
 					Namespace: "myns",
 				},
-				Status: machinev1.MachineStatus{},
+				Status: machinev1beta1.MachineStatus{},
 			},
-			ExpectedMachine: machinev1.Machine{
-				Status: machinev1.MachineStatus{
+			ExpectedMachine: machinev1beta1.Machine{
+				Status: machinev1beta1.MachineStatus{
 					Addresses: []corev1.NodeAddress{
 						corev1.NodeAddress{
 							Address: "192.168.1.1",
@@ -1215,12 +1224,12 @@ func TestUpdateMachineStatus(t *testing.T) {
 					},
 				},
 			},
-			Machine: &machinev1.Machine{
+			Machine: &machinev1beta1.Machine{
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "mymachine",
 					Namespace: "myns",
 				},
-				Status: machinev1.MachineStatus{
+				Status: machinev1beta1.MachineStatus{
 					Addresses: []corev1.NodeAddress{
 						corev1.NodeAddress{
 							Address: "192.168.1.1",
@@ -1233,8 +1242,8 @@ func TestUpdateMachineStatus(t *testing.T) {
 					},
 				},
 			},
-			ExpectedMachine: machinev1.Machine{
-				Status: machinev1.MachineStatus{
+			ExpectedMachine: machinev1beta1.Machine{
+				Status: machinev1beta1.MachineStatus{
 					Addresses: []corev1.NodeAddress{
 						corev1.NodeAddress{
 							Address: "192.168.1.1",
@@ -1251,15 +1260,15 @@ func TestUpdateMachineStatus(t *testing.T) {
 		{
 			// machine status unchanged
 			Host: &bmh.BareMetalHost{},
-			Machine: &machinev1.Machine{
+			Machine: &machinev1beta1.Machine{
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "mymachine",
 					Namespace: "myns",
 				},
-				Status: machinev1.MachineStatus{},
+				Status: machinev1beta1.MachineStatus{},
 			},
-			ExpectedMachine: machinev1.Machine{
-				Status: machinev1.MachineStatus{},
+			ExpectedMachine: machinev1beta1.Machine{
+				Status: machinev1beta1.MachineStatus{},
 			},
 		},
 	}
@@ -1284,7 +1293,7 @@ func TestUpdateMachineStatus(t *testing.T) {
 			Name:      tc.Machine.Name,
 			Namespace: tc.Machine.Namespace,
 		}
-		machine := machinev1.Machine{}
+		machine := machinev1beta1.Machine{}
 		c.Get(context.TODO(), key, &machine)
 
 		if &tc.Machine != nil {
@@ -1292,7 +1301,7 @@ func TestUpdateMachineStatus(t *testing.T) {
 				Name:      tc.Machine.Name,
 				Namespace: tc.Machine.Namespace,
 			}
-			machine := machinev1.Machine{}
+			machine := machinev1beta1.Machine{}
 			c.Get(context.TODO(), key, &machine)
 
 			if tc.Machine.Status.Addresses != nil {
@@ -1308,7 +1317,7 @@ func TestUpdateMachineStatus(t *testing.T) {
 
 func TestApplyMachineStatus(t *testing.T) {
 	scheme := runtime.NewScheme()
-	clusterapis.AddToScheme(scheme)
+	machinev1beta1.AddToScheme(scheme)
 
 	addr1 := corev1.NodeAddress{
 		Type:    "InternalIP",
@@ -1321,18 +1330,18 @@ func TestApplyMachineStatus(t *testing.T) {
 	}
 
 	testCases := []struct {
-		Machine               *machinev1.Machine
+		Machine               *machinev1beta1.Machine
 		Addresses             []corev1.NodeAddress
 		ExpectedNodeAddresses []corev1.NodeAddress
 	}{
 		{
 			// Machine status updated
-			Machine: &machinev1.Machine{
+			Machine: &machinev1beta1.Machine{
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "machine1",
 					Namespace: "myns",
 				},
-				Status: machinev1.MachineStatus{
+				Status: machinev1beta1.MachineStatus{
 					Addresses: []corev1.NodeAddress{},
 				},
 			},
@@ -1341,12 +1350,12 @@ func TestApplyMachineStatus(t *testing.T) {
 		},
 		{
 			// Machine status unchanged
-			Machine: &machinev1.Machine{
+			Machine: &machinev1beta1.Machine{
 				ObjectMeta: metav1.ObjectMeta{
 					Name:      "machine1",
 					Namespace: "myns",
 				},
-				Status: machinev1.MachineStatus{
+				Status: machinev1beta1.MachineStatus{
 					Addresses: []corev1.NodeAddress{addr1, addr2},
 				},
 			},
@@ -1376,7 +1385,7 @@ func TestApplyMachineStatus(t *testing.T) {
 			Name:      tc.Machine.Name,
 			Namespace: tc.Machine.Namespace,
 		}
-		machine := machinev1.Machine{}
+		machine := machinev1beta1.Machine{}
 		c.Get(context.TODO(), key, &machine)
 
 		if tc.Machine.Status.Addresses != nil {
@@ -1391,7 +1400,7 @@ func TestApplyMachineStatus(t *testing.T) {
 
 func TestNodeAddresses(t *testing.T) {
 	scheme := runtime.NewScheme()
-	clusterapis.AddToScheme(scheme)
+	machinev1beta1.AddToScheme(scheme)
 
 	nic1 := bmh.NIC{
 		IP: "192.168.1.1",
@@ -1496,3 +1505,336 @@ func TestNodeAddresses(t *testing.T) {
 		}
 	}
 }
+
+func TestDeleteOfBareMetalHostDeletesMachine(t *testing.T) {
+	scheme := runtime.NewScheme()
+	machinev1beta1.AddToScheme(scheme)
+	bmoapis.AddToScheme(scheme)
+
+	testCases := []struct {
+		CaseName              string
+		Host                  *bmh.BareMetalHost
+		Machine               *machinev1beta1.Machine
+		ExpectedMachineExists bool
+	}{
+		{
+			CaseName: "machine should not be deleted",
+			Host: &bmh.BareMetalHost{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "myhost1",
+					Namespace: "myns",
+				},
+				Spec: bmh.BareMetalHostSpec{
+					ConsumerRef: &corev1.ObjectReference{
+						Name:       "mymachine1",
+						Namespace:  "myns",
+						Kind:       "Machine",
+						APIVersion: machinev1beta1.SchemeGroupVersion.String(),
+					},
+				},
+				Status: bmh.BareMetalHostStatus{
+					HardwareProfile: "dell",
+					Provisioning: bmh.ProvisionStatus{
+						State: "ready",
+					},
+				},
+			},
+			Machine: &machinev1beta1.Machine{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "mymachine1",
+					Namespace: "myns",
+					Annotations: map[string]string{
+						HostAnnotation: "myns/myhost1",
+					},
+				},
+				Status: machinev1beta1.MachineStatus{},
+			},
+			ExpectedMachineExists: true,
+		},
+		{
+			CaseName: "machine should be deleted",
+			Host: &bmh.BareMetalHost{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "myhost2",
+					Namespace: "myns",
+				},
+				Spec: bmh.BareMetalHostSpec{
+					ConsumerRef: &corev1.ObjectReference{
+						Name:       "mymachine2",
+						Namespace:  "myns",
+						Kind:       "Machine",
+						APIVersion: machinev1beta1.SchemeGroupVersion.String(),
+					},
+				},
+				Status: bmh.BareMetalHostStatus{
+					HardwareProfile: "dell",
+					Provisioning: bmh.ProvisionStatus{
+						State: "deleting",
+					},
+				},
+			},
+			Machine: &machinev1beta1.Machine{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "mymachine2",
+					Namespace: "myns",
+					Annotations: map[string]string{
+						HostAnnotation: "myns/myhost2",
+					},
+				},
+				Status: machinev1beta1.MachineStatus{},
+			},
+			ExpectedMachineExists: false,
+		},
+	}
+
+	for _, tc := range testCases {
+		c := fakeclient.NewFakeClientWithScheme(scheme)
+		if tc.Machine != nil {
+			c.Create(context.TODO(), tc.Machine)
+		}
+		if tc.Host != nil {
+			c.Create(context.TODO(), tc.Host)
+		}
+		actuator, err := NewActuator(ActuatorParams{
+			Client: c,
+		})
+		if err != nil {
+			t.Error(err)
+		}
+
+		err = actuator.Update(context.TODO(), tc.Machine)
+		if err != nil {
+			t.Errorf("unexpected error %v", err)
+		}
+
+		if &tc.Machine != nil {
+			key := client.ObjectKey{
+				Name:      tc.Machine.Name,
+				Namespace: tc.Machine.Namespace,
+			}
+			machine := machinev1beta1.Machine{}
+
+			err = c.Get(context.TODO(), key, &machine)
+			if tc.ExpectedMachineExists == false {
+				if !errors.IsNotFound(err) {
+					t.Errorf("Expected machine to not exist but received error: %v", err)
+				}
+			} else {
+				result, err := actuator.Exists(context.TODO(), tc.Machine)
+				if err != nil {
+					t.Error(err)
+				}
+				if tc.ExpectedMachineExists != result {
+					t.Errorf("ExpectedMachineExists: %v, found %v", tc.ExpectedMachineExists, result)
+				}
+			}
+		}
+	}
+}
+
+func TestRemediation(t *testing.T) {
+	machine, machineNamespacedName := getMachine("machine1")
+	host, hostNamespacedName := getBareMetalHost("host1")
+	node, nodeNamespacedName := getNode("node1")
+	linkMachineAndNode(machine, node)
+	host.Status.PoweredOn = true
+
+	//starting test with machine that needs remediation
+	machine.Annotations = make(map[string]string)
+	machine.Annotations[HostAnnotation] = host.Namespace + "/" + host.Name
+	machine.Annotations[externalRemediationAnnotation] = ""
+
+	scheme := runtime.NewScheme()
+	machinev1beta1.AddToScheme(scheme)
+	bmoapis.AddToScheme(scheme)
+	corev1.AddToScheme(scheme)
+
+	c := fakeclient.NewFakeClientWithScheme(scheme)
+
+	actuator, err := NewActuator(ActuatorParams{
+		Client: c,
+	})
+	if err != nil {
+		t.Error(err)
+	}
+
+	c.Create(context.TODO(), machine)
+	c.Create(context.TODO(), host)
+	c.Create(context.TODO(), node)
+
+	err = actuator.Update(context.TODO(), machine)
+	if err != nil {
+		t.Errorf("unexpected error %v", err)
+	}
+
+	host = &bmh.BareMetalHost{}
+	c.Get(context.TODO(), hostNamespacedName, host)
+	if !hasPowerOffRequestAnnotation(host) {
+		t.Log("Expected reboot annotation on the host but none found")
+		t.Fail()
+	}
+
+	host.Status.PoweredOn = false
+	c.Update(context.TODO(), host)
+
+	err = actuator.Update(context.TODO(), machine)
+	if err != nil {
+		t.Errorf("unexpected error %v", err)
+	}
+
+	machine = &machinev1beta1.Machine{}
+	c.Get(context.TODO(), machineNamespacedName, machine)
+
+	if _, exists := machine.Annotations[poweredOffForRemediation]; !exists {
+		t.Log("Expected powered-off-for-remediation annotation to exist on machine but none found")
+		t.Fail()
+	}
+
+	err = actuator.Update(context.TODO(), machine)
+	if err == nil {
+		t.Errorf("expected a requeue err but err was nil")
+	} else {
+		switch err.(type) {
+		case *machineapierrors.RequeueAfterError:
+			break
+		default:
+			t.Errorf("unexpected error %v", err)
+		}
+	}
+
+	node = &corev1.Node{}
+	err = c.Get(context.TODO(), nodeNamespacedName, node)
+	if !errors.IsNotFound(err) {
+		t.Log("Expected node to be deleted")
+		t.Fail()
+	}
+
+	err = actuator.Update(context.TODO(), machine)
+	if err != nil {
+		t.Errorf("unexpected error %v", err)
+	}
+
+	host = &bmh.BareMetalHost{}
+	c.Get(context.TODO(), hostNamespacedName, host)
+
+	if hasPowerOffRequestAnnotation(host) {
+		t.Log("Expected reboot annotation to be removed but it's still there")
+		t.Fail()
+	}
+
+	//simulate power on and node that comes up again
+	host.Status.PoweredOn = true
+	c.Update(context.TODO(), host)
+
+	node, _ = getNode("node1")
+	linkMachineAndNode(machine, node)
+	c.Create(context.TODO(), node)
+	c.Update(context.TODO(), machine)
+
+	err = actuator.Update(context.TODO(), machine)
+	if err != nil {
+		t.Errorf("unexpected error %v", err)
+	}
+
+	machine = &machinev1beta1.Machine{}
+	c.Get(context.TODO(), machineNamespacedName, machine)
+
+	if len(machine.Annotations) > 0 {
+		if _, exists := machine.Annotations[externalRemediationAnnotation]; exists {
+			t.Log("Expected external remediation annotation to be removed but it's still there")
+			t.Fail()
+		}
+
+		if _, exists := machine.Annotations[poweredOffForRemediation]; exists {
+			t.Log("Expected powered-off-for-remediation annotation to be removed but it's still there")
+			t.Fail()
+		}
+	}
+
+	//make sure nothing happens after remediation
+	err = actuator.Update(context.TODO(), machine)
+	if err != nil {
+		t.Errorf("unexpected error %v", err)
+	}
+	err = actuator.Update(context.TODO(), machine)
+	if err != nil {
+		t.Errorf("unexpected error %v", err)
+	}
+
+	host = &bmh.BareMetalHost{}
+	c.Get(context.TODO(), hostNamespacedName, host)
+
+	if hasPowerOffRequestAnnotation(host) {
+		t.Log("Expected reboot annotation to be removed but it's still there, maybe reboot loops?")
+		t.Fail()
+	}
+}
+
+func getNode(name string) (*corev1.Node, types.NamespacedName) {
+	node := &corev1.Node{ObjectMeta: metav1.ObjectMeta{
+		Name:      name,
+		Namespace: testRemediationNamespace,
+	},
+		TypeMeta: metav1.TypeMeta{
+			Kind:       "Node",
+			APIVersion: corev1.SchemeGroupVersion.String(),
+		}}
+
+	nodeNamespacedName := types.NamespacedName{
+		Namespace: node.ObjectMeta.Namespace,
+		Name:      node.ObjectMeta.Name,
+	}
+
+	return node, nodeNamespacedName
+}
+
+func getBareMetalHost(name string) (*bmh.BareMetalHost, types.NamespacedName) {
+	host := &bmh.BareMetalHost{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      name,
+			Namespace: "default",
+		},
+		TypeMeta: metav1.TypeMeta{
+			Kind:       "BareMetalHost",
+			APIVersion: bmh.SchemeGroupVersion.String(),
+		},
+		Spec:   bmh.BareMetalHostSpec{},
+		Status: bmh.BareMetalHostStatus{},
+	}
+
+	hostNamespacedName := types.NamespacedName{
+		Namespace: host.ObjectMeta.Namespace,
+		Name:      host.ObjectMeta.Name,
+	}
+
+	return host, hostNamespacedName
+}
+
+func getMachine(name string) (*machinev1beta1.Machine, types.NamespacedName) {
+	machine := &machinev1beta1.Machine{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      name,
+			Namespace: testRemediationNamespace,
+		},
+		TypeMeta: metav1.TypeMeta{
+			Kind:       "Machine",
+			APIVersion: machinev1beta1.SchemeGroupVersion.String(),
+		},
+		Status: machinev1beta1.MachineStatus{},
+	}
+
+	machineNamespacedName := types.NamespacedName{
+		Namespace: machine.ObjectMeta.Namespace,
+		Name:      machine.ObjectMeta.Name,
+	}
+
+	return machine, machineNamespacedName
+}
+
+func linkMachineAndNode(machine *machinev1beta1.Machine, node *corev1.Node) {
+	machine.Status.NodeRef = &corev1.ObjectReference{
+		Kind:      "Node",
+		Namespace: testRemediationNamespace,
+		Name:      node.Name,
+	}
+}
